@@ -22,16 +22,20 @@ pub struct Cursor<'a> {
 }
 
 impl<'a> Cursor<'a> {
+    #[allow(clippy::suspicious_map)]
     pub fn advance(&self, amt: usize) -> Cursor<'a> {
         let mut column_start: Option<usize> = None;
         Cursor {
             rest: &self.rest[amt..],
             off: self.off + (amt as u32),
-            line: self.line + self.rest[..amt]
-                .char_indices()
-                .filter(|&(_, ref x)| *x == '\n')
-                .map(|(i, _)| column_start = Some(i))
-                .count() as u32,
+            line: self.line
+                + self.rest[..amt]
+                    .char_indices()
+                    .filter(|&(_, ref x)| *x == '\n')
+                    .map(|(i, _)| {
+                        column_start = Some(i);
+                    })
+                    .count() as u32,
             column: match column_start {
                 None => self.column + (amt as u32),
                 Some(i) => (amt - i) as u32 - 1,
@@ -106,7 +110,7 @@ pub fn whitespace(input: Cursor) -> PResult<()> {
             }
         }
         match bytes[i] {
-            b' ' | 0x09...0x0d => {
+            b' ' | 0x09..=0x0d => {
                 i += 1;
                 continue;
             }
@@ -119,11 +123,7 @@ pub fn whitespace(input: Cursor) -> PResult<()> {
                 }
             }
         }
-        return if i > 0 {
-            Ok((s, ()))
-        } else {
-            Err(LexError { line: s.line })
-        };
+        return if i > 0 { Ok((s, ())) } else { Err(LexError { line: s.line }) };
     }
     Ok((input.advance(input.len()), ()))
 }
@@ -169,19 +169,12 @@ fn is_whitespace(ch: char) -> bool {
 
 #[inline]
 fn is_ident_start(c: char) -> bool {
-    ('a' <= c && c <= 'z')
-        || ('A' <= c && c <= 'Z')
-        || c == '_'
-        || (c > '\x7f' && UnicodeXID::is_xid_start(c))
+    c.is_ascii_alphabetic() || c == '_' || (c > '\x7f' && UnicodeXID::is_xid_start(c))
 }
 
 #[inline]
 fn is_ident_continue(c: char) -> bool {
-    ('a' <= c && c <= 'z')
-        || ('A' <= c && c <= 'Z')
-        || c == '_'
-        || ('0' <= c && c <= '9')
-        || (c > '\x7f' && UnicodeXID::is_xid_continue(c))
+    c.is_ascii_alphanumeric() || c == '_' || (c > '\x7f' && UnicodeXID::is_xid_continue(c))
 }
 
 pub fn symbol(input: Cursor) -> PResult<&str> {
@@ -304,6 +297,7 @@ pub fn cooked_byte_string(mut input: Cursor) -> PResult<()> {
 pub fn raw_string(input: Cursor) -> PResult<()> {
     let mut chars = input.char_indices();
     let mut n = 0;
+    #[allow(clippy::while_let_on_iterator)] //chars is used in the next loop
     while let Some((byte_offset, ch)) = chars.next() {
         match ch {
             '"' => {
@@ -393,8 +387,8 @@ fn backslash_x_char<I>(chars: &mut I) -> bool
 where
     I: Iterator<Item = (usize, char)>,
 {
-    next_ch!(chars @ '0'...'7');
-    next_ch!(chars @ '0'...'9' | 'a'...'f' | 'A'...'F');
+    next_ch!(chars @ '0'..='7');
+    next_ch!(chars @ '0'..='9' | 'a'..='f' | 'A'..='F');
     true
 }
 
@@ -402,8 +396,8 @@ fn backslash_x_byte<I>(chars: &mut I) -> bool
 where
     I: Iterator<Item = (usize, u8)>,
 {
-    next_ch!(chars @ b'0'...b'9' | b'a'...b'f' | b'A'...b'F');
-    next_ch!(chars @ b'0'...b'9' | b'a'...b'f' | b'A'...b'F');
+    next_ch!(chars @ b'0'..=b'9' | b'a'..=b'f' | b'A'..=b'F');
+    next_ch!(chars @ b'0'..=b'9' | b'a'..=b'f' | b'A'..=b'F');
     true
 }
 
@@ -412,9 +406,9 @@ where
     I: Iterator<Item = (usize, char)>,
 {
     next_ch!(chars @ '{');
-    next_ch!(chars @ '0'...'9' | 'a'...'f' | 'A'...'F');
+    next_ch!(chars @ '0'..='9' | 'a'..='f' | 'A'..='F');
     loop {
-        let c = next_ch!(chars @ '0'...'9' | 'a'...'f' | 'A'...'F' | '_' | '}');
+        let c = next_ch!(chars @ '0'..='9' | 'a'..='f' | 'A'..='F' | '_' | '}');
         if c == '}' {
             return true;
         }
